#ifndef LINE_H
#define LINE_H

#include "Shape.h"

// The Line class
class Line : public Shape {
  private:
    // The coordinates of the line endpoints: (x1,y1) - (x2,y2)
    double x1,x2,y1,y2;
    // Draw a handle at the endpoints of the line or not.
    int handle_status;

  public:
    // The default line constructor.
    Line();
    // Line constructor
    Line(double r, double g, double b, double thickness, double x1,
	 double y1, double x2, double y2);
    Line(Line *line);

    // The following functions return each of the line coordinate components.
    double get_x1() { return x1; }
    double get_y1() { return y1; }
    double get_x2() { return x2; }
    double get_y2() { return y2; }

    // Set the line coordinates.
    void setcoords(double x1, double y1, double x2, double y2);
    
    // Check if the mouse is on the line or not.
    // Return: 1 if it is near the first endpoint.
    //         2 if it is near the second endpoint.
    //         3 if it is near the inside of the line.
    //         0 if it is not on near the line.
    int mouse_on(double mousex, double mousey, int handlesize);

    // Draw a handle at coordinates x,y of specified size.
    void draw_handle(int x, int y, int handlesize);
    // Set handle status.
    void set_handle_status(int new_status) { handle_status = new_status; }
    // Return handle status.
    int handles_on() { return handle_status; }
    // Draws an arrow at the end of one endpoint.
    void draw_arrow(double mag);

    // Draw the line.
    void draw(double mag);
    // Get the line shape type.
    int get_type();
    // Read a line from a file stream f.
    void read(fstream& f);
    // Write a line from to file stream f.
    void write(fstream& f);
};

#endif
